
/*
 * MGL graphics library
 * Copyright (C) 1999-2000, ing. Marek Gayer

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <mgl.h>

extern int mainOption;
extern mglFont *mainFont;
extern int mainPaused;
extern int mainSpeed;

extern void mainMakeCheckImage (unsigned char *map, int size, int segs);

static mglTexture *cubeTexture, *lightTexture, *wallTexture;
static mglTexture *chessTextureLinear, *chessTextureNearest, *stencilMask, *stencilTexture, *stencilScene;
static mglImage *chessImage, *lightImage, *stencilMaskImage, *stencilImage;

static float angle;

static void buildLightPatern (unsigned char *mem, int size)
{
  int i,j;
  float fXCenter, fYCenter;
  float fA;
  float fXD, fYD, fDist;

  fXCenter = size/2.0f;
  fYCenter = size/2.0f;

  for (j=0; j<size; j++)
  {
    fYD = fYCenter - ((float)j);
    for (i=0; i<size; i++)
    {
      fXD = fXCenter - ((float)i);
      fDist = (fXD*fXD + fYD*fYD) / (fXCenter*fXCenter);

      if (fDist > 1.0f) fDist = 1.0f;
        fA = 1.0f - fDist;

      if ((j < (size>>1)) && (i < (size>>1)))
      {
        mem[0] = 0xff;
        mem[1] = 0x00;
        mem[2] = 0x00;
      }
      else
        if (j < (size>>1))
        {
          mem[0] = 0x00;
          mem[1] = 0xff;
          mem[2] = 0x00;
        }
        else
          if (i < (size>>1))
          {
            mem[0] = 0x00;
            mem[1] = 0x00;
            mem[2] = 0xff;
          }
          else
          {
            mem[0] = 0xff;
            mem[1] = 0xff;
            mem[2] = 0x00;
          }
        mem[3] = fA * 255.0f;
        mem += 4;
    }
  }
}

void figuresInit (void)
{
  int w;
  int h;
  int i;
  unsigned char c;

  angle = 0;

  if (!cubeTexture)
  {
    cubeTexture = mglLoadTexture ("../data/demo/cube.tga", 0, GL_LINEAR);
    wallTexture = mglLoadTexture ("../data/demo/wall.tga", 0, GL_LINEAR);

    chessImage = mglCreateImage (32, 32, 1);
    mainMakeCheckImage (chessImage->data, 32, 16);
    chessImage->type = GL_LUMINANCE;
    chessTextureLinear = mglMakeTexture (chessImage, GL_LINEAR);
    chessTextureNearest = mglMakeTexture (chessImage, GL_NEAREST);

    /*
    multiImage = mglCreateImage (128, 128, 1);
    mainMakeCheckImage (multiImage->data, 128, 16);
    multiImage->type = GL_LUMINANCE;
    */

    lightImage = mglCreateImage (128, 128, 4);
    buildLightPatern (lightImage->data, 128);
    lightImage->type = GL_RGBA;
    /*
    for (i = 0; i < 128 * 128; i++)
    {
      if (lightImage->data [4*i+3] == 0)
      {
        lightImage->data [4*i+0] = 0;
        lightImage->data [4*i+1] = 0;
        lightImage->data [4*i+2] = 0;
        lightImage->data [4*i+3] = 255;
      }
    }
    */
    lightTexture = mglMakeTexture (lightImage, GL_LINEAR);

    stencilImage = mglLoadImage ("../data/demo/stencil.tga", 1);
    stencilTexture = mglMakeTexture (stencilImage, GL_LINEAR);
    stencilScene = mglLoadTexture ("../data/demo/lake.tga", 0, GL_LINEAR);

    w = stencilTexture->image.w;
    h = stencilTexture->image.h;
    stencilMaskImage = mglCreateImage (w, h, 1);
    stencilMaskImage->type = GL_LUMINANCE;
    stencilMaskImage->xrev = stencilTexture->image.xrev;
    stencilMaskImage->yrev = stencilTexture->image.yrev;
    for (i = 0; i < w *h; i++)
    {
      c = stencilImage->data [4*i+3];
//      if (c)
        //c = 255;
      stencilMaskImage->data [i] = c;
    }
    stencilMask = mglMakeTexture (stencilMaskImage, GL_NEAREST);

    mglDeleteImage (stencilMaskImage);
    mglDeleteImage (stencilImage);
    mglDeleteImage (chessImage);
    mglDeleteImage (lightImage);
  }
}


void figuresTexture (void)
{
  int r;
  int len = 500;
  int sx = 500;
  int sy = 600;
  int lineSize;

  mglClearScreen (mglWhite);

  r = sqrt (2 * len * len) / 2;
  mglSetLineColors (mglGreen);
  lineSize = mglMaxLineSize;
  if (lineSize > 5)
    lineSize = 5;
  mglSetLineSize (lineSize);
  mglDrawLine (sx - r, sy, sx - r, sy - r/2);
  mglDrawLine (sx + r, sy, sx + r, sy - r/2);
  mglDrawLine (sx, sy - r, sx, sy - 3*r/2);
  mglDrawLine (sx + r, sy - r/2, sx, sy - 3*r/2);
  mglDrawLine (sx - r, sy - r/2, sx, sy - 3*r/2);

  mglRotationAngle = 45;
  mglDrawTexturedRectangle (cubeTexture, sx - len/2, sy - len/2, 500, 500);
  mglRotationAngle = 0;
}

void figuresAlfa (void)
{
  mglColor c;

  mglClearScreen (mglWhite);
  mglAlphaOn ();

  c = mglCeleste;
  mglSetTriangleColors (c);
  mglDrawFilledTriangle (400, 100, 100, 700, 700, 700);

  c = mglPink;
  c.a = 0.333;
  mglSetRectangleColors (c);
  mglDrawFilledRectangle (300, 400, 500, 500);
}

void figuresLines (void)
{
  mglClearScreen (mglWhite);

  mglSetLineColors (mglBlue);
  mglSetLineSize (1);

  mglSmoothOn ();
  mglDrawLine (600, 900, 900, 100);

  mglSmoothOff ();
  mglDrawLine (100, 900, 400, 100);
}

void figuresInterpolate (void)
{
  mglClearScreen (mglCeleste);

  mglTriangleColor [0] = mglWhite;
  mglTriangleColor [1] = mglBlack;
  mglTriangleColor [2] = mglWhite;

  mglDrawFilledTriangle (500, 100, 100, 900, 900, 900);
}

void figuresBilinear (mglTexture *tex)
{
  mglSetTextureColors (mglWhite);

  mglTextureZoomX = 1 / (float) mainSpeed;
  mglTextureZoomY = 1 / (float) mainSpeed;

  if (mglFps)
  {
    angle += (float)30 / mglFps;
    if (angle > 360)
      angle -= 360;
  }
  mglRotationAngle = angle;

  mglDrawTexturedRectangle (tex, -1000, -1000, 3000, 3000);
  mglRotationAngle = 0;

  mglTextureZoomX = mglTextureZoomY = 1;
}



static int option;
static char *title;

static void setOption (void)
{
  mglSetWindowTitle (title);

  if (option == mainOption)
    return;
  option = mainOption;
}

void figuresMultitexture (void)
{
  mglTextureZoomX = 1 / (float) mainSpeed;
  mglTextureZoomY = 1 / (float) mainSpeed;
  mglDrawTexturedRectangle (chessTextureLinear, 0, 0, 1000, 1000);

  mglTextureZoomX = mglTextureZoomY = 1;

  if (mglFps)
  {
    angle += (float)30 / mglFps;
    if (angle > 360)
      angle -= 360;
  }

  mglRotationAngle = angle;
  mglDrawTexturedRectangle (lightTexture, 0, 0, 1000, 1000);
  mglRotationAngle = 0;
}

void figuresStencilMask (void)
{
  mglDrawTexturedRectangle (stencilMask, 0, 0, 1000, 1000);
}

void figuresStencilTexture (void)
{
  mglClearScreen (mglBlack);

  mglDrawTexturedRectangle (stencilTexture, 0, 0, 1000, 1000);
}

void figuresStencilScene (void)
{
  mglDrawTexturedRectangle (stencilScene, 0, 0, 1000, 1000);
}

void figuresStencilResult (void)
{
  mglDrawTexturedRectangle (stencilScene, 0, 0, 1000, 1000);
  mglDrawTexturedRectangle (stencilTexture, 0, 0, 1000, 1000);
}

void figuresFonts (int mode)
{
  mglSetTextureColors (mglWhite);

  mglDrawTexturedRectangle (wallTexture, 0, 0, 1000, 1000);

  mglTextureZoomX = 2 * (mainSpeed);
  mglTextureZoomY = 2 * (mainSpeed);

  if (mode)
    mglAlphaOn ();
  else
    mglAlphaOff ();

  mglSetTextureColors (mglRed);
  mglPuts (mainFont, "MGL", 70, 333);

  mglAlphaOn ();

  mglSetTextureZoom (1);
}


void figuresDisplay (void)
{
  mglSetTextureColors (mglWhite);

  switch (mainOption)
  {
    case 1:
      title = "Mapovn textur";
      figuresTexture ();
      break;
    case 2:
      title = "Mchn pruhlednch vrstev";
      figuresAlfa ();
      break;
    case 3:
      title = "Vyhlazovn hran";
      figuresLines ();
      break;
    case 4:
      title = "Interpolace barev";
      figuresInterpolate ();
      break;
    case 5:
      title = "Bez bilinern interpolace";
      figuresBilinear (chessTextureNearest);
      break;
    case 6:
      title = "S bilinern interpolac";
      figuresBilinear (chessTextureLinear);
      break;
    case 7:
      title = "Multitexturing";
      figuresMultitexture ();
      break;
    case 8:
      title = "Maska sablony 1/4";
      figuresStencilMask ();
      break;
    case 9:
      title = "Maska sablony 2/4";
      figuresStencilTexture ();
      break;
    case 10:
      title = "Maska sablony 3/4";
      figuresStencilScene ();
      break;
    case 11:
      title = "Maska sablony 4/4";
      figuresStencilResult ();
      break;
    case 12:
      title = "Texturovan fonty 1/2";
      figuresFonts (0);
      break;
    case 13:
      title = "Texturovan fonty 2/2";
      figuresFonts (1);
      break;
    default:
      break;
  }

  setOption ();
}
