
/*
 * MGL graphics library
 * Copyright (C) 1999-2000, ing. Marek Gayer

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <stdio.h>

#ifdef _WIN32
#include <windows.h>
//#include <crtdbg.h>
#include <winuser.h>
#endif

#include <mgl.h>

static char *mglErrorArray [256];

char *mglErrorType;

FILE *mglNoteFile;

void initErrorArray (void)
{
  int i;

  for (i = 0; i < 256; i++)
    mglErrorArray [i] = "MGL_ERROR_UNKNOWN";

  mglErrorArray [MGL_ERROR_UNKNOWN] = "MGL_ERROR_UNKNOWN";

  mglErrorArray [MGL_ERROR_COMPILER_OPTIONS] = "MGL_ERROR_COMPILER_OPTIONS";

  mglErrorArray [MGL_ERROR_NOT_ENOUGH_MEMORY] = "MGL_ERROR_NOT_ENOUGH_MEMORY";

  mglErrorArray [MGL_ERRORS_OPENGL_CALLS] = "MGL_ERRORS_OPENGL_CALLS";

  mglErrorArray [MGL_ERROR_USER] = "MGL_ERROR_USER";

  mglErrorArray [MGL_ERROR_NO_OPENGL_DRIVERS] = "MGL_ERROR_NO_OPENGL_DRIVERS";
  mglErrorArray [MGL_ERROR_NO_GLU_DRIVERS] = "MGL_ERROR_NO_GLU_DRIVERS";
  mglErrorArray [MGL_ERROR_UNKNOWN_OPENGL_VERSION] = "MGL_ERROR_UNKNOWN_OPENGL_VERSION";

  mglErrorArray [MGL_ERROR_SUBTEXTURE_COORDINATES] = "MGL_ERROR_SUBTEXTURE_COORDINATES";
  mglErrorArray [MGL_ERROR_UNKNOWN_TEXTURE_MODE] = "MGL_ERROR_UNKNOWN_TEXTURE_MODE";
  mglErrorArray [MGL_ERROR_TOO_BIG_TEXTURE] = "MGL_ERROR_TOO_BIG_TEXTURE";
  mglErrorArray [MGL_ERROR_TEXTURE_SIZE] = "MGL_ERROR_TEXTURE_SIZE";
  mglErrorArray [MGL_ERROR_MISSING_MIPMAP] = "MGL_ERROR_MISSING_MIPMAP";

  mglErrorArray [MGL_ERROR_OPENING_FILE] = "MGL_ERROR_OPENING_FILE";
  mglErrorArray [MGL_ERROR_WRITING_FILE] = "MGL_ERROR_WRITING_FILE";
  mglErrorArray [MGL_ERROR_CREATING_FILE] = "MGL_ERROR_CREATING_FILE";

  mglErrorArray [MGL_ERROR_READING_IMAGE_DATA] = "MGL_ERROR_READING_IMAGE_DATA";
  mglErrorArray [MGL_ERROR_READING_TGA_HEADER] = "MGL_ERROR_READING_TGA_HEADER";
  mglErrorArray [MGL_ERROR_READING_SGI_HEADER] = "MGL_ERROR_READING_SGI_HEADER";
  mglErrorArray [MGL_ERROR_READING_BMP_HEADER] = "MGL_ERROR_READING_BMP_HEADER";

  mglErrorArray [MGL_ERROR_UNSUPPORTED_IMAGE] = "MGL_ERROR_UNSUPPORTED_IMAGE";
  mglErrorArray [MGL_ERROR_UNSUPPORTED_TGA_IMAGE] = "GL_ERROR_UNSUPPORTED_TGA_IMAGE";
  mglErrorArray [MGL_ERROR_UNSUPPORTED_BMP_IMAGE] = "MGL_ERROR_UNSUPPORTED_BMP_IMAGE";
  mglErrorArray [MGL_ERROR_UNSUPPORTED_SGI_IMAGE] = "MGL_ERROR_UNSUPPORTED_SGI_IMAGE";

  mglErrorArray [MGL_ERROR_FONT_DEFINITION_FILE] = "MGL_ERROR_FONT_DEFINITION_FILE";
}





void mglDefaultErrorHandler (int errorNumber, char *errorMessage)
{
  char s [64000];

  sprintf (s, "The following error (type %s) occured:\n"
              "\n"
              "%s.\n"
              "\n"
              "Application has been terminated.\n", mglErrorType, errorMessage);

	#ifdef _WIN32
    /*
		#if defined _DEBUG
			#if defined _MSC_VER
				_CrtDbgReport( _CRT_ERROR, NULL, 0, "MGL debug error", errorMessage);
			#else
				MessageBox (NULL, errorMessage, "MGL error", MB_ICONSTOP );
			#endif
		#else
    */
			MessageBox (NULL, s, "MGL error", MB_ICONSTOP );
//		#endif
	#else
		printf (s);
    printf (".\n");
 	#endif
}

static mglErrorHandlerType mglErrorHandler = mglDefaultErrorHandler;

void mglSetErrorHandler (mglErrorHandlerType func)
{
  if (func)
  {
    mglErrorHandler = func;
  }
  else
  {
    mglErrorHandler = mglDefaultErrorHandler;
  }
}


void mglError (int errorNumber, char *errorMessage)
{
  initErrorArray ();
  mglErrorType = mglErrorArray [errorNumber];

  mglErrorHandler (errorNumber, errorMessage);

  if (mglNoteFile)
    fclose (mglNoteFile);

  exit (1);
}


static int fileAppend (char *name, char *string)
{
  FILE *file;

  if ((file = fopen (name, "a+t")) == NULL)
  {
    return -1;
  }

  if (fputs (string, file) == EOF)
  {
    fclose (file);
    return -2;
  }
  fclose (file);
  return 0;
}


void mglNote (char *s)
{
  #ifdef _DEBUG
    /*
    if (!mglNoteFile)
    {
      remove ("mglnotes.txt");
      mglNoteFile = fopen ("mglnotes.txt", "wt");
      if (mglNoteFile == NULL)
        mglError (MGL_ERROR_UNKNOWN, "Debug error. Cannot open mglnotes.txt for writing.\n");
    }
    fputs (s, mglNoteFile);
    */
	if (mglLog)
	{
		if (fileAppend ("mgl_log.txt", s))
		{
		mglError (MGL_ERROR_UNKNOWN, "Debug error. Cannot open mgl_log.txt for writing.\n");
		}
	}
  #endif
}


//-----------------------------------------------------------------------------

void mglMessage (char *title, char *message)
{
	#ifdef _WIN32
		MessageBox( NULL, message, title, MB_OK );
	#else
		printf ("%s\n\n%s\n", title, message);
	#endif
}


//-----------------------------------------------------------------------------
#ifdef _DEBUG

void mglTestOpenGLError(void) {
	// jen pri ladeni, jinak zdrzuje


	int ec;
	char* es;
  char e [100];

	if ( ( ec = glGetError() ) != GL_NO_ERROR ) {
		switch ( ec ) {
			case GL_NO_ERROR: es = "GL_NO_ERROR"; // pro jistotu
			break;
			case GL_INVALID_ENUM: es = "GL_INVALID_ENUM";
			break;
			case GL_INVALID_VALUE: es = "GL_INVALID_VALUE";
			break;
			case GL_INVALID_OPERATION: es = "GL_INVALID_OPERATION";
			break;
			case GL_STACK_OVERFLOW: es = "GL_STACK_OVERFLOW";
			break;
			case GL_STACK_UNDERFLOW: es = "GL_STACK_UNDERFLOW";
			break;
			case GL_OUT_OF_MEMORY: es = "GL_OUT_OF_MEMORY";
			break;
			default: es = "unknown OpenGL error";
			break;
		}
    strcpy (e, "Oops! I screwed up my OpenGL calls somewhere - glGetError () returned ");
    strcat (e, es);
		mglError(MGL_ERRORS_OPENGL_CALLS, e );
	}
}

#endif // _DEBUG

